/*==================================================================*/
/*                                                                  */
/*    RBLAST.H - Standard header for Sound Blaster support.         */
/*                                                                  */
/*==================================================================*/

#ifndef       _RBLAST_

#define       _RBLAST_

#define       FM_MUSIC_ADDR           0x08
#define       FM_MUSIC_STATUS         0x08
#define       FM_MUSIC_DATA           0x09

#define       DSP_RESET               0x06
#define       DSP_DATA                0x0a
#define       DSP_COMMAND             0x0c
#define       DSP_STATUS              0x0c
#define       DSP_AVAILABLE           0x0e

#define       SOUND_CHARACTERISTIC    0x00
#define       KEY_SCALING             0x01
#define       ATTACK_DECAY            0x02
#define       SUSTAIN_RELEASE         0x03
#define       WAVE_SELECT             0x04
#define       FEEDBACK                0x05

#define       KEY_ON_BASE             0xb0
#define       FREQ_BASE               0xa0
#define       MELODY_CTRL             0xbd

#define       VOICE_ON                0x20
#define       VOICE_OFF               0x00
#define       RHYTHM_ON               0x20

typedef struct
{
    char           marker[20];          /*  Creative Voice File marker  */
    unsigned int   hdrlen;              /*  Offset of first block       */
    unsigned int   version;             /*  Coded Version number        */
    unsigned int   vercomp;             /*  Complement of version+1234H */
}   VOC_HDR;

typedef struct
{
    char           marker[4];           /*  RocSoft sample file marker  */
    unsigned char  tconst;              /*  Time constant               */
    unsigned char  ptype;               /*  Pack type                   */
    unsigned int   len;                 /*  Length of sample (bytes)    */
    unsigned char  *data;               /*  Sample itself               */
}   RSAMPLE;

typedef struct
{
    char             marker[4];         /*  CTMF  marker  */
    unsigned int     version;
    unsigned int     inst_offset;
    unsigned int     music_offset;
    unsigned int     ticks_per_qtr;
    unsigned int     ticks_per_sec;
    unsigned int     title_offset;
    unsigned int     composer_offset;
    unsigned int     remks_offset;
    unsigned char    channels[16];
    unsigned int     inst_count;
    unsigned int     tempo;
}   CMF_HEADER;

typedef struct
{
    unsigned char    block[16];
}   CMF_INSTRUMENT;

int   SB_enable        ( void );
int   SB_disable       ( void );
int   SB_play          ( RSAMPLE *s );
int   SB_pause         ( int pause );
int   SB_speaker       ( int state );
void  SB_wait          ( void );
void  SB_abort         ( void );
int   SB_load_sample   ( char *file, RSAMPLE *sample );
int   SB_free_sample   ( RSAMPLE *sample );
int   SB_play_cmf      ( char *cmfname, int volperc, int repeat );

#endif
