/*
**  Function prototypes for 'rgraph' graphics library routines.
**  (c) R. Loscalzo, 1992.
*/

#ifndef     _RGRAPH_

#define     _RGRAPH_

#define     MODE_40x25       0x01      /* Graphics mode numbers */
#define     MODE_80x25       0x03
#define     MODE_320x200x4   0x04
#define     MODE_640x200x2   0x06
#define     MODE_80x25_Mono  0x07
#define     MODE_320x200x16  0x0d
#define     MODE_640x200x16  0x0e
#define     MODE_640x350x16  0x10
#define     MODE_640x480x16  0x12

#define     MODE_320x200x256 0x7ff1
#define     MODE_320x240x256 0x7ff2
#define     MODE_360x480x256 0x7ff3

#define     MODE_IS_256      ((VS.mode&0x7ff0)==0x7ff0)


#define     BLACK            0x00      /* Colour manifest constants */
#define     BLUE             0x01
#define     GREEN            0x02
#define     CYAN             0x03
#define     RED              0x04
#define     MAGENTA          0x05
#define     BROWN            0x06
#define     L_GREY           0x07
#define     GREY             0x08
#define     L_BLUE           0x09
#define     L_GREEN          0x0a
#define     L_CYAN           0x0b
#define     L_RED            0x0c
#define     L_MAGENTA        0x0d
#define     L_YELLOW         0x0e
#define     WHITE            0x0f

#define     BORDER           0x00       /* Action constants for rect & ellipse functions */
#define     FILL             0x01

#define     RETAIN_DISPLAY   0x80       /* Used when setting a new mode */

#define     CURSOR_OFF       setcurs( 32, 0 )
#define     ATTR(b,f)        ((((b)&0x07)<<4)|(f))
#define     FLASH            0x80

#define     ROM_8x14         2          /* ROM font parameter types */
#define     ROM_8x8          3
#define     ROM_8x16         6

#define     CONVEX           0          /* 3d button type constants */
#define     CONCAVE          1

#define     EGA_PALETTE_SIZE 17         /* Palette data sizes */
#define     VGA_PALETTE_SIZE 768

#define     SPLIT_PAGE       -1         /* Split screen page code  */

typedef struct
{
        int           x, y;             /* 2d point coordinates */
} POINT;

typedef struct
{
        unsigned char *table;           /* Ptr to character font table       */
        int           height;           /* Height of font in pixels          */
} FONT;

struct VIDEO_MODE
{
        int          mode;              /* Current mode number               */
        int          mx, my;            /* Maximum x & y coordinates         */
        int          cols;              /* Number of cols that make up a row */
        unsigned int page[10];          /* Page segment addresses            */
        int          maxpage;           /* Number of valid pages             */
        unsigned int spage;             /* Split screen page segment addr    */
        unsigned int slines;            /* Number of lines on split screen   */
} VS;

struct SCREEN_SCROLL
{
        int          xoff, yoff;        /* Top left of virtual window        */
        int          xlim, ylim;        /* Max x & y offsets allowed         */
}   SCR;

typedef enum                            /* Palette type data type            */
{
        EGA_PALETTE=0, VGA_PALETTE
} PALETTE_TYPE;

typedef struct                          /* Palette data type itself          */
{
        PALETTE_TYPE   type;            /* Palette type                      */
        unsigned char  *pal;            /* Pointer to palette data           */
} PALETTE;


/*  General functions  */

char adapter  ( void );
void setmode  ( int mode );
int  getmode  ( void );
void setpage  ( int page );
int  getpage  ( void );
void showpage ( int page );
int  getcols  ( void );
void setpos   ( int col, int row, int page );
void getpos   ( int *col, int *row, int page );
void border   ( char color );
void setbkgnd ( int color );
void cls      ( int attribute );
void setcurs  ( int start, int stop );
void getcurs  ( int *start, int *stop );
int  clip     ( int *x1, int *y1, int *x2, int *y2, int xmin, int ymin, int xmax, int ymax );

/*  Main graphics 'primitives'  */

void line     ( int x1, int y1, int x2, int y2, int color, int page );
void ellipse  ( int x, int y, int radius, int color, float aspect, int type, int page );
void rect     ( int x1, int y1, int x2, int y2, int color, int type, int page );
void polygon  ( POINT *pts, int num, int xoff, int yoff, int colour, int fill, int page );
void floodfill( int sx, int sy, int border, int fcolor, int page );
void rrect    ( int x1, int y1, int x2, int y2, int r, int colour, int fill, int page );
void arc      ( int x, int y, int r, int sang, int eang, int colour, int fill, float aspect, int page );
void setpixel ( int x, int y, int color, int page );
int  rdpixel  ( int x, int y, int page );


/*  Palette support functions  */

#define  allocpal(p,t)           ((((p)->pal=malloc((((p)->type=t)==EGA_PALETTE)?EGA_PALETTE_SIZE:VGA_PALETTE_SIZE))==NULL)?NULL:(p))
#define  freepal(p)              free( (p)->pal )
int      palregidx( PALETTE_TYPE type, int colour );
int      setpal   ( PALETTE *p, int colour );
int      setpals  ( PALETTE *p );
int      getpal   ( PALETTE *p, int colour );
int      getpals  ( PALETTE *p );
int      loadpals ( char *file, PALETTE *p );
int      savepals ( char *file, PALETTE *p );
void     adjpal   ( PALETTE *p, int colour, unsigned char *rgb );
void     rdpal    ( PALETTE *p, int colour, unsigned char *rgb );


/*  Special 'effects' functions  */

void box3d    ( int x1, int y1, int x2, int y2, int bw, int sun, int shdw, int fill, int type, int page );
int  splitscr ( int line );
int  fadeout  ( PALETTE *p, int delay );
int  fadein   ( PALETTE *p, int delay );


/*  Video memory manipulation routines  */

void vmemcopy ( int xf, int yf, int pfrom, int xt, int yt, int pto, int h, int w );
int  create_virtual_screen( int width, int height );
int  set_virtual_window   ( int x, int y, int page, int vsync );


/*  Font support functions  */

int  rom_font ( FONT _far *font, int ftype );
int  loadfont ( char *fontname, FONT _far *font );
int  savefont ( char *fontname, FONT _far *font );
int  fadefont ( FONT *norm, FONT *fade );


/*  Graphics font printing and string entry  */

void gprint   ( unsigned char *msg, int x, int y, int color, int scale, FONT _far *font, int page );
void gprinti  ( int num, int x, int y, int color, int scale, FONT _far *font, int page );
void gprintf  ( char *msg, int x, int y, int fgnd, int bgnd, FONT _far *font, int page );
void gprintfi ( int num, int x, int y, int fgnd, int bgnd, FONT _far *font, int page );
int  ggetstr  ( int x, int y, char *string, int len, int fcolor, int bcolor, FONT *font );

#endif
