/*==================================================================*/
/*                                                                  */
/*    RINOUT.H - Standard header for I/O based functions.           */
/*                                                                  */
/*==================================================================*/

#ifndef  _RINOUT_

#define  _RINOUT_

#define  F1KEY        0x13b        /* Manifest constants for extended keys */
#define  F2KEY        0x13c
#define  F3KEY        0x13d
#define  F4KEY        0x13e
#define  F5KEY        0x13f
#define  F6KEY        0x140
#define  F7KEY        0x141
#define  F8KEY        0x142
#define  F9KEY        0x143
#define  F10KEY       0x144
#define  HOMEKEY      0x147
#define  ENDKEY       0x14f
#define  PAGEUPKEY    0x149
#define  PAGEDNKEY    0x151
#define  INSERTKEY    0x152
#define  DELETEKEY    0x153
#define  UPARROWKEY   0x148
#define  DNARROWKEY   0x150
#define  LFARROWKEY   0x14b
#define  RIARROWKEY   0x14d

#define  RETURNKEY    0x0d         /* Manifest constants for normal keys   */
#define  SPACEBAR     0x20
#define  BACKSPACEKEY 0x08
#define  ESCAPEKEY    0x1b

#define  MOUSE_INT    0x33         /* Manifest constants for mouse control */
#define  L_BUTTON     0x01
#define  R_BUTTON     0x02

#define  MSE_MOVED    0x01         /* Mouse event bit masks for MOUSETRAP  */
#define  MSE_LB_DOWN  0x02
#define  MSE_LB_UP    0x04
#define  MSE_RB_DOWN  0x08
#define  MSE_RB_UP    0x10

#define  FIRE_BUTTON_1  0x01       /* Bit masks for joystick fire buttons  */
#define  FIRE_BUTTON_2  0x02

/*  Prototypes for simple graphics screen output  */

void printc  ( char ch, int color, int page );
void prints  ( char *string, int color, int page );
void printi  ( int num, int color, int page );
void printsi ( char *string, int num, int color, int page );
void printsl ( char *string, int size, int bed, int color, int page );
void printsr ( char *string, int size, int bed, int color, int page );
void printil ( int num, int size, int bed, int color, int page );
void printir ( int num, int size, int bed, int color, int page );

/*  Event timing prototypes  */

long timer   ( void );

/*  Prototypes for keyboard input functions  */

int  inkey   ( int mode );
int  getkey  ( int mode );
int  ctrlkey ( void );
int  getstr  ( int x, int y, char *string, int len, int color );

/*  Prototypes for simple printer output  */

char pstatus ( void );
char pprintc ( char ch );
void pprint  ( int num, int ch, ... );
void pprints ( char *string );
void pprinti ( int num );
void pprintsi( char *string, int num );
void pprintsl( char *string, int size, int bed );
void pprintsr( char *string, int size, int bed );
void pprintil( int num, int size, int bed );
void pprintir( int num, int size, int bed );

/*  Prototypes for standard mouse service functions  */

int  chkmouse( void );
int  mstatus ( int *buttons, int *xpos, int *ypos );
void mouseon ( void );
void mouseoff( void );
void mouselim( int xmin, int ymin, int xmax, int ymax );
void mousepos( int x, int y );
int  moushape( SPRITE *spr, SPRITE *mask, int hotx, int hoty );

/*  Prototype, etc, for 'pseudo' mouse facility  */

typedef (*MOUSETRAP) ( int event, int b, int x, int y, int mx, int my );

int  enable_pseudo_mouse ( SPRITE *shape, MOUSETRAP trap );
int  disable_pseudo_mouse( void );

/*  Prototypes for joystick interface  */

int  joystick_1( int *xc, int *yc );
int  joystick_2( int *xc, int *yc );

#endif
