/*==================================================================*/
/*                                                                  */
/*    RSPRITE.H - Header file for sprite, image & object support.   */
/*                                                                  */
/*==================================================================*/

#ifndef  _RSPRITE_

#define  _RSPRITE_

#ifndef  FALSE
#define  FALSE           0x00
#define  TRUE            0x01
#endif

#define  OWRTE           0x00         /* Constants for putspr mode */
#define  XOR             0x01
#define  OR              0x02
#define  AND             0x03

#define  ADD_OBJECT      0x00         /* Object action types */
#define  ACTIVE_OBJECT   0x01
#define  DELETE_OBJECT   0x02

#define  COLLIDE         0x00
#define  NO_COLLIDE      0x01

#define  NO_COLLISION      -1

#ifndef  NULL
#define  NULL            0x00L
#endif

struct  IMAGE_VIEWPORT                /* Top left & bottom right of image viewport */
{
        int x1, y1, x2, y2;
}       IVW;

typedef struct
{
        int                h;               /* Pixel height of sprite */
        int                w;               /* Pixel width of sprite  */
        unsigned char      *buff;           /* Pointer to data buffer */
} SPRITE;

typedef struct                              /* Collision box struct   */
{
        int x[4];                           /* Top left/bottom right  */
        int y[4];                           /* Coords for 2 boxes     */
} CBOX;

typedef enum
{
        GENERAL_IMAGE, OVERLAY_16, OVERLAY_256, VRAM_OVERLAY
} IMAGE_TYPE;

typedef struct
{
        int                h;               /* Pixel height of image  */
        int                w;               /* Pixel width of image   */
        unsigned char      *buff;           /* Ptr to data buffer     */
        CBOX               *rect;           /* 2 collision rectangles */
        IMAGE_TYPE         type;            /* Image type flag        */
} IMAGE;

typedef                                     /*  Object transformation function!  */
  int TRANSFORM( void *object, int *idx );
typedef                                     /*  Object automation function!      */
  int AUTOMATON( void *object, int *x, int *y );

typedef struct                              /* Series of frames to be animated! */
{
        IMAGE              *img;            /* Image data pointer               */
        int                frames;          /* Number of frames within above    */
        AUTOMATON          *mover;          /* Object mover function            */
        TRANSFORM          *morph;          /* Object frame metamorphosis func  */
        int                collide;         /* Collision detection boolean      */
        int                delay;           /* Frame delay count                */
} CARTOON;

typedef                                     /* Background update function */
  void  BGND_DRAWER( int xoff,int yoff,int bpage,int page );

typedef                                     /* Page switching function */
  void  PAGE_SWITCHER( int page );

typedef struct
{
        int                status;              /* Action indicator for updobj        */
        int                pri;                 /* Image level (priority) number      */
        int                idx;                 /* Cartoon frame index number (curr)  */
        int                x1,y1,x2,y2;         /* Top left & bottom right of object  */
        int                delay;               /* Current frame delay count          */
        int                dflag;               /* Flag set when object can be rmvd   */
        CARTOON            *toon;               /* Pointer to animation frames        */
} OBJECT;

typedef struct
{
        int                h_tiles, v_tiles;    /* Horiz & vert tile dimensions */
        int                size;                /* Pixel size of tile (square)  */
        int                *map;                /* Tile grid for tile indices   */
        int                num;                 /* Number of tile types         */
        SPRITE             *tiles;              /* Tile images                  */
        IMAGE_TYPE         type;                /* Type of image                */
} SCROLL_MAP;

void  putspr   ( SPRITE *sprite, int x, int y, int mode, int page );
int   getspr   ( SPRITE *sprite, int x, int y, int width, int height, int page );
void  movespr  ( SPRITE *sprite, int oldx, int oldy, int newx, int newy, int blankcolor, int page );
int   rdspr    ( SPRITE *sprite, int handle );
int   wtspr    ( SPRITE *sprite, int handle );
int   rdcbox   ( CBOX   *rect, int handle );
int   wtcbox   ( CBOX   *rect, int handle );
int   loadspr  ( SPRITE spr[], char *fname );
int   savespr  ( SPRITE spr[], char *fname, int num );
int   loadcbox ( CBOX   box[], char *fname );
int   savecbox ( CBOX   box[], char *fname, int num );
void  waitVR   ( void );

void  copyvid  ( int x1, int y1, int x2, int y2, int pfrom, int pto );

int           set_vram_buff( unsigned long bytes );
unsigned long vram_free    ( void );
unsigned int  vram_alloc   ( int width, int height );
void          putvram      ( SPRITE *spr, unsigned int vbuff );
int           spr2overlay  ( SPRITE *spr, char **buff, IMAGE_TYPE type );

int   imgvwport( int x1, int y1, int x2, int y2 );
int   genimg   ( SPRITE *spr, IMAGE  *img, IMAGE_TYPE type, CBOX *rect );
int   putimg   ( IMAGE  *img, int x, int y, int page );

int   make_cartoon      ( CARTOON **toon,SPRITE spr[],IMAGE_TYPE type,CBOX bbox[],int frames,
                          int delay, int collide, TRANSFORM *morph, AUTOMATON *mover );
void  remove_cartoon    ( CARTOON  *toon );
int   initialise_objects( int num, BGND_DRAWER *bgndupdate, PAGE_SWITCHER *switcher );
int   remove_objects    ( void );
int   add_object        ( int pri, CARTOON *toon, int idx, int x, int y );
int   delete_object     ( int pri );
int   replace_object    ( int pri, CARTOON *toon, int idx );
int   update_objects    ( int xoff, int yoff, int bpage );
int   check_collision   ( int pri );
int   object_active     ( int pri );
int   object_index      ( int pri );

int   Cycle_Morph       ( void *object, int *idx );
int   PingPong_Morph    ( void *object, int *idx );
int   Burst_Morph       ( void *object, int *idx );
int   Retro_Morph       ( void *object, int *idx );
int   Static_Morph      ( void *object, int *idx );

void  Selective_Replace ( int xoff, int yoff, int bpage, int page );
void  Complete_Replace  ( int xoff, int yoff, int bpage, int page );

int   loadmap      ( SCROLL_MAP *smap, char *mapdef, char *mapdata, char *tiledata, IMAGE_TYPE type );
void  freemap      ( SCROLL_MAP *smap );
int   changemap    ( SCROLL_MAP *smap, int xt, int yt, int tilenum );
int   placemap     ( SCROLL_MAP *smap, int xt, int yt, int xoff, int yoff, int page );
int   drawmap      ( SCROLL_MAP *smap, int xoff, int yoff, int page );

#endif
